@@ -5,6 +5,8 @@ import subprocess
 
 # TODO:
 # see, if you can activate virtualenv
+# for JavaFX, check if the distro have rsync installed if not install it
+# (requires for update_files.sh) and 'entr' too
 
 class SetProject:
     """ General Project setup class """
@@ -232,12 +234,16 @@ class LangJava(SetProject):
 
     def __init__(self, name='new-project'):
         super().__init__(name)
+        self.dsrc = self.path / 'src'
+        self.dbin = self.path / 'bin'
+        self.packageName = None
+        self.packageDir = None
 
     def __writeToFiles(self):
         for gotFile in self.dsrc.rglob("*.java"):
             with open(gotFile, 'w') as file:
                 if gotFile.stem == "Main":
-                    file.writelines(f"""package com.{self.packageName.stem};
+                    file.writelines(f"""package com.{self.packageName.replace('/', '.')};
 
 public class {gotFile.stem} {{
 \tpublic static void main(String ...args) {{
@@ -245,10 +251,10 @@ public class {gotFile.stem} {{
 \t}}
 }}""")
                 else:
-                    file.writelines(f"""package com.{self.packageName.stem};
+                    file.writelines(f"""package com.{self.packageName.replace('/', '.')};
 
 class {gotFile.stem} {{
-        /** code */
+\t\t/** code */
 }}""")
 
     def setup(self):
@@ -256,9 +262,9 @@ class {gotFile.stem} {{
         self.dbin = self.path / 'bin'
         self.packageName = input("Package Name [if not provided, default will be hostname]: ")
         if not self.packageName:
-            self.packageName = self.dsrc / ('com') / uname()[1]
+            self.packageDir = self.dsrc / ('com') / uname()[1]
         else:
-            self.packageName = self.dsrc / ('com') / self.packageName
+            self.packageDir = self.dsrc / ('com') / self.packageName
         className = list()
         try:
             count = int(input('Number of class files(except Main.java): '))
@@ -266,17 +272,21 @@ class {gotFile.stem} {{
             print("Enter correct values: " + ve)
         else:
             for x in range(count):
-                className.append((input(str(x + 1) + ": ")).title())
+                gotClassName = input(str(x + 1) + ": ")
+                if gotClassName.islower():
+                    className.append(gotClassName.title())
+                else:
+                    className.append(gotClassName)
             choice = input("Separate Main.java? [y/n]: ")
             if choice == 'y':
                 className.append('Main')
             if len(className) > 0:
                 self.dbin.mkdir()
-                self.packageName.mkdir(parents=True)
+                self.packageDir.mkdir(parents=True)
                 for classfile in className:
-                    Path.touch(self.packageName / (classfile + ".java"))
+                    Path.touch(self.packageDir / (classfile + ".java"))
                 if choice == 'y':
-                    Path.touch(self.packageName / 'Main.java')
+                    Path.touch(self.packageDir / 'Main.java')
                 LangJava.__writeToFiles(self)
             else:
                 print("Error: There must atleast on class file")
@@ -285,6 +295,123 @@ class {gotFile.stem} {{
                 exit(2)
 
 
+class LangJavaFx(LangJava):
+    """further project setup for JavaFx(suitable for terminal development)"""
+
+    def __init__(self, name="new_project"):
+        super().__init__(name)
+        self.fxmlFile = None
+        self.cssFile = None
+        self.scenebuilderPos = "/opt/SceneBuilder/app"
+        self.updateScript = self.dsrc / "update_files.sh"
+
+    def __writeToFiles(self):
+        for gotFile in self.dsrc.rglob("*.java"):
+            with open(gotFile, "w") as file:
+                if gotFile.stem == "Main":
+                    if not self.fxmlFile:
+                        file.writelines(f"""package com.{self.packageName.replace('/', '.')}
+
+import javafx.application.Application;
+import javafx.stage.Stage;
+import javafx.scene.Scene;
+import javafx.scene.layout.GridPane;
+
+public class {gotFile.stem} extends Application {{
+
+\t@Override
+\tpublic void start(Stage primaryStage) throws Exception {{
+\t\tGridPane root = new GridPane();
+\t\tprimaryStage.setTitle("{self.projName}");
+\t\tprimaryStage.setScene(new Scene(root, "400", "400"));
+\t\tprimaryStage.show();
+\t}}
+\tpublic static void main(String ...args) {{
+\t\tlaunch(args);
+\t}}
+}}""")
+                    else:
+                        file.writelines(f"""package com.{self.packageName.replace('/', '.')}
+
+import javafx.application.Application;
+import javafx.stage.Stage;
+import javafx.scene.Scene;
+import javafx.scene.Parent;
+import javafx.fxml.FXMLLoader;
+
+public class {gotFile.stem} extends Application {{
+
+\t@Override
+\tpublic void start(Stage primaryStage) throws Exception {{
+\t\tParent root = new FXMLLoader(getClass().getResource("/com/{self.packageName}/{self.fxmlFile}"));
+\t\tprimaryStage.setTitle("{self.projName}");
+\t\tprimaryStage.setScene(new Scene(root, "400", "400"));
+\t\tprimaryStage.show();
+\t}}
+\tpublic static void main(String ...args) {{
+\t\tlaunch(args);
+\t}}
+}}""")
+                # going to be Controller or some other java files
+                else:
+                    file.writelines(f"""package com.{self.packageName.replace('/', '.')}
+
+public class {gotFile.stem} {{
+\t/** code here */
+}}""")
+        # basic setup for fxml files
+        for gotFile in self.dsrc.rglob("*.fxml"):
+            with open(gotFile, 'w') as file:
+                file.writelines("""<?import javafx.geometry.Insets?>
+<?import javafx.scene.layout.GridPane?>
+
+<GridPane fx:controller="" xmlns:fx="http://javafx.com/fxml">
+\t<!-- code here -->
+</GridPane>""")
+        # basic setup for css files
+        for gotFile in self.dsrc.rglob("*.css"):
+            with open(gotFile, 'w') as file:
+                file.write("/* provide styling here */")
+        if self.scenebuilderPos:
+            if Path(self.scenebuilderPos).is_dir():
+                with open(Path(self.scenebuilderPos + '/' + self.fxmlFile), 'w') as file:
+                    file.writelines("""<?import javafx.geometry.Insets?>
+<?import javafx.scene.layout.GridPane?>
+
+<GridPane fx:controller="" xmlns:fx="http://javafx.com/fxml">
+\t<!-- code here -->
+</GridPane>""")
+        with open(Path(self.updateScript), 'w') as file:
+            file.writelines(f"""#!/usr/bin/env bash
+
+# file: {Path(self.fxmlFile).stem}.fxml
+# source: {self.scenebuilderPos}
+# desitination: src and bin(module)
+rsync -av {self.scenebuilderPos}/{Path(self.fxmlFile).stem}.fxml {self.dsrc}/com/{self.packageName}
+rsync -av {self.scenebuilderPos}/{Path(self.fxmlFile).stem}.fxml {self.dbin}/com.{self.packageName.replace('/', '.')}/com/{self.packageName}
+
+# file: {Path(self.fxmlFile).stem}.fxml
+# source: src
+# desitnation: scenebuilder pos. and bin(module)
+rsync -a {self.dsrc}/{self.packageName}/{Path(self.fxmlFile).stem}.fxml {self.scenebuilderPos}
+rsync -a {self.dsrc}/{self.packageName}/{Path(self.fxmlFile).stem}.fxml {self.dbin}/com.{self.packageName.replace('/', '.')}/com/{self.packageName}
+
+# provide further sync details here""")
+
+    def doFurtherSetup(self):
+        self.fxmlFile = input("Enter the fxml file name[if not, leave blank]: ")
+        self.cssFile = input("Enter the css file name[if not, leave blank]:")
+        print("Enter scenebuilder position")
+        enteredSceneBuilderPos = input("[default: \"/opt/SceneBuilder/app\" | if not, leave blank]: ")
+        if enteredSceneBuilderPos:
+            self.scenebuilderPos = enteredSceneBuilderPos
+        if self.fxmlFile:
+            Path.touch(self.packageDir / (self.fxmlFile + ".fxml"))
+        if self.cssFile:
+            Path.touch(self.packageDir / (self.cssFile + ".css"))
+        (self.dbin / ('com.' + self.packageName.replace('/', '.'))).mkdir()  # create module
+        LangJavaFx.__writeToFiles(self)
+
 class LangServlet(SetProject):
     """ further project setup for Java Servlet languages """
 
